import json
import logging
import os

import requests
from django.conf import settings
from django.core.management.base import BaseCommand
from fec_dbsync.models import GeneralAccount, RoboChatView, RoboIdtView, RoboEitView
from django.db.models import Q

from fec_dbsync.constants import collections_journey_name_config

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = 'Command for post due.'

    def get_config(self, name):
        url = 'http://localhost:8000/integrations/api/config/?name=' + name

        try:
            response = requests.get(url)
            response = json.loads(response.content, encoding='utf-8')
            if 'response' in response and response['response']:
                return  response['response']
            return None
        except Exception:
            return  None

    def handle(self, *args, **options):
        """

        :param args:
        :param options:
        :return:
        """

        verbosity = int(options['verbosity'])

        root_logger = logging.getLogger('')

        if verbosity > 1:
            root_logger.setLevel(logging.DEBUG)

        post_due_records = GeneralAccount.objects.filter(DPD_CNTR__gt=0)

        collections_payload = []

        for application in post_due_records:
            emi = 0

            emi_records_idt = RoboIdtView.objects.filter(Q(LOAN_ACCT_NUM=application.LOAN_ACCT_NUM),
                                                         Q(DMD_FLOW_ID='PRDEM') | Q(DMD_FLOW_ID='INDEM'))
            for record in emi_records_idt:
                emi = emi + max(record.DMD_AMT - record.TOT_ADJ_AMT, 0)

            penalty_amount = 0

            penalty_records_idt = RoboIdtView.objects.filter(LOAN_ACCT_NUM=application.LOAN_ACCT_NUM,
                                                             DMD_FLOW_ID='PIDEM')
            for record in penalty_records_idt:
                penalty_amount = penalty_amount + max(record.DMD_AMT - record.TOT_ADJ_AMT, 0)

            penalty_records_eit = RoboEitView.objects.filter(LOAN_ACCT_NUM=application.LOAN_ACCT_NUM)
            for record in penalty_records_eit:
                penalty_amount = penalty_amount + max(record.PENAL_BOOKED_AMOUNT_DR - record.PENAL_INTEREST_AMOUNT_DR,
                                                      0)

            processing_fee = 0

            processing_records_chat = RoboChatView.objects.filter(LOAN_ACCT_NUM=application.LOAN_ACCT_NUM,
                                                                  CHARGE_TYPE='MISC2')
            for record in processing_records_chat:
                processing_fee = processing_fee + max(record.USER_CALC_CHRGE_AMT,
                                                      record.SYS_CALC_CHRGE_AMT) \
                                 - record.CHRGE_AMT_COLLECTED - record.CHRGE_WAIVED

            single_payload = {
                'loanContractNumber': application.LOAN_ACCT_NUM,
                'dpd': application.DPD_CNTR,
                'outstandingLoanAmount': abs(application.CURRENT_BAL),
                'penalty': penalty_amount,
                'processingFee': processing_fee,
                'emi': emi
            }

            collections_payload.append(single_payload)

        login_url = settings.WORKFLOW_BASE_URL + "/login/credentials/"
        headers = {
            'Content-Type': "application/json",
            'Cache-Control': "no-cache"
        }
        data = {
            "username": os.environ.get('FLOWABLE_WORKFLOW_COLLECTION_USER', 'collections_admin'),
            "password": os.environ.get('FLOWABLE_WORKFLOW_COLLECTION_PASSWORD', 'test')
        }
        response = requests.post(login_url, data=json.dumps(data), headers=headers)
        response = response.json()
        if not response.get('status') == 200 or not response.get('data'):
            return {"status": False, "message": "Can't Login"}
        auth_token = response.get('data').get('bpm_token')

        # journey_name = self.get_config(collections_journey_name_config)

        # Hardcoding this until config API is pushed.
        journey_name = "test"
        
        if not journey_name or len(collections_payload) == 0:
            return "No records to process."

        payload = {
            'journeyName': journey_name,
            'collections': collections_payload
        }

        URL = settings.WORKFLOW_BASE_URL + "/collections/initiate"

        logger.info("Request payload: " + str(payload))

        headers = {'content-type': 'application/json', "Authorization": "Bearer " + auth_token}

        response = requests.post(URL, data=json.dumps(payload), headers=headers)

        logger.info("Response : " + str(response.content))



import requests
import json
from datetime import date, timedelta, datetime

from django.core.management.base import BaseCommand
from django.conf import settings
from django.db import connection
import logging
from fec_dbsync.models import GeneralAccount

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = 'Command for sending the notifications for users'

    def handle(self, *args, **options):
        """

        :param args:
        :param options:
        :return:
        """

        verbosity = int(options['verbosity'])
        root_logger = logging.getLogger('')

        if verbosity > 1:
            root_logger.setLevel(logging.DEBUG)

        response = requests.get(settings.WORKFLOW_BASE_URL + "/config?name=feCreditBusinessDate")
        try:
            today = response.json().get('data').get('value')
            today = datetime.strptime(today, '%d-%m-%Y')
        except:
            today = datetime.now()
        current_date = today
        logger.info("Current date: " + current_date.strftime('%Y-%m-%d'))

        query_date = current_date + timedelta(days=5)
        dmd_query_date = current_date - timedelta(days=270)
        dmd_query_date = dmd_query_date.strftime('%Y-%m-%d')
        cursor = connection.cursor()
        s = 'select fec_dbsync_robolrsview.LOAN_ACCT_NUM, fec_dbsync_robolrsview.NEXT_DMD_DATE,fec_dbsync_robolrsview.FLOW_AMT, COUNT(CASE WHEN fec_dbsync_roboidtview.DMD_AMT - fec_dbsync_roboidtview.TOT_ADJ_AMT <= 0 and fec_dbsync_roboidtview.LAST_ADJ_DATE > "{}" THEN 1 ELSE NULL END) as months from fec_dbsync_robolrsview LEFT OUTER JOIN fec_dbsync_roboidtview ON fec_dbsync_roboidtview.LOAN_ACCT_NUM = fec_dbsync_robolrsview.LOAN_ACCT_NUM group by fec_dbsync_robolrsview.LOAN_ACCT_NUM,fec_dbsync_robolrsview.NEXT_DMD_DATE,fec_dbsync_robolrsview.FLOW_AMT'
        cursor.execute(s.format(dmd_query_date))
        data  = {
                    "contractId" : "",
                    "dueDate" : "",
                    "dueAmount" : "",
                    "sendSMS" : True,
                    "sendNotification" : True,
                    "sendEmail" : True,
                    "smsTemplateKey" : "",
                    "notificationTemplateKey" : "",
                    "emailTemplateKey" : ""
                }
        URL = settings.WORKFLOW_BASE_URL + "/notify/"
        for row in cursor:
            contract_number = row[0]

            gam_object = GeneralAccount.objects.get(LOAN_ACCT_NUM=contract_number)

            if gam_object.DPD_CNTR > 0:
                continue

            count  = row[3]
            diff = row[1] - current_date.date()
            data["contractId"] = row[0]
            data["dueDate"] = row[1].strftime('%d-%m-%Y')
            data["dueAmount"] = row[2]
            logger.debug ("Count: " + str(count))
            logger.debug ("Diff: " + str(diff))
            logger.debug ("Current_Date: " + current_date.strftime('%Y-%m-%d'))
            if count>=9:
                if diff.days == 3 :
                    data["smsTemplateKey"] = "DPD_3"
                    data["notificationTemplateKey"] = "DPD_3"
                    data["emailTemplateKey"] = "DPD_3"
                elif diff.days == 0 :
                    data["smsTemplateKey"] = "DPD_0"
                    data["notificationTemplateKey"] = "DPD_0"
                    data["emailTemplateKey"] = "DPD_0"
                else:
                    continue

            if count<=9 and count>=6:
                if diff.days == 5 :
                    data["smsTemplateKey"] = "DPD_5"
                    data["notificationTemplateKey"] = "DPD_5"
                    data["emailTemplateKey"] = "DPD_5"
                elif diff.days == 0 :
                    data["smsTemplateKey"] = "DPD_0"
                    data["notificationTemplateKey"] = "DPD_0"
                    data["emailTemplateKey"] = "DPD_0"
                else:
                    continue

            if count<=6 and count>=3:
                if diff.days == 5:
                    data["smsTemplateKey"] = "DPD_5"
                    data["notificationTemplateKey"] = "DPD_5"
                    data["emailTemplateKey"] = "DPD_5"
                elif diff.days == 3:
                    data["smsTemplateKey"] = "DPD_3"
                    data["notificationTemplateKey"] = "DPD_3"
                    data["emailTemplateKey"] = "DPD_3"
                elif diff.days == 0:
                    data["smsTemplateKey"] = "DPD_0"
                    data["notificationTemplateKey"] = "DPD_0"
                    data["emailTemplateKey"] = "DPD_0"
                else:
                    continue

            if count<=3 :
                if diff.days == 5:
                    data["smsTemplateKey"] = "DPD_5"
                    data["notificationTemplateKey"] = "DPD_5"
                    data["emailTemplateKey"] = "DPD_5"
                elif diff.days == 3:
                    data["smsTemplateKey"] = "DPD_3"
                    data["notificationTemplateKey"] = "DPD_3"
                    data["emailTemplateKey"] = "DPD_3"
                elif diff.days == 0:
                    data["smsTemplateKey"] = "DPD_0"
                    data["notificationTemplateKey"] = "DPD_0"
                    data["emailTemplateKey"] = "DPD_0"
                else:
                    continue
            logger.info("Request payload: " + str(data))
            headers = {'content-type':'application/json'}

            response = requests.post(URL, data=json.dumps(data), headers=headers)
            logger.info("Response : " + str(response.content))


# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-05-30 23:24
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('fec_dbsync', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='generalaccount',
            old_name='Acct_status',
            new_name='ACCT_STATUS',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='acid',
            new_name='ACID',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='charge_off_flag',
            new_name='CHRGE_OFF_FLG',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='cif_id',
            new_name='CIF_ID',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='current_balance',
            new_name='CURRENT_BAL',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='debt_group',
            new_name='DEBT_GROUP',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='disb_date',
            new_name='DISB_DATE',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='dpd_ason_date',
            new_name='DPD_ASON_DATE',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='dpd_cntr',
            new_name='DPD_CNTR',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='inst_start_date',
            new_name='INST_START_DATE',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='int_dmd_os',
            new_name='INT_DMD_OS',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='lms_schm_code',
            new_name='LMS_SCHM_CODE',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='loan_account_number',
            new_name='LOAN_ACCT_NUM',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='maturity_date',
            new_name='MATURITY_DATE',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='past_due_flag',
            new_name='PAST_DUE_FLG',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='payoff_date',
            new_name='PAYOFF_DATE',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='pms_schm_desc',
            new_name='PMS_SCHM_DESC',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='pms_schm_id',
            new_name='PMS_SCHM_ID',
        ),
        migrations.RenameField(
            model_name='generalaccount',
            old_name='prin_dmd_os',
            new_name='PRIN_DMD_OS',
        ),
    ]



from django.conf.urls import url

from fec_dbsync import views


urlpatterns = [
    url(r'^lms-data/', views.GetLmsData.as_view(), name='get-lms-data'),
    url(r'^robo-active-apps-count/', views.GetRoboActiveApps.as_view(), name='get-robo-active-apps-count'),
    url(r'^get_repayment_schedule/', views.GetRepaymentSchedule.as_view(), name='get-repayment-schedule'),
    url(r'^generate_mis_report/', views.GenerateMISReport.as_view()),
    url(r'^get_repayment_schedule/',views.GetRepaymentSchedule.as_view(), name='get-repayment-schedule'),
    url(r'^collections_repayment/',views.CollectionsRepayment.as_view(), name='collections-repayment'),
    url(r'^collections_statement/', views.CollectionsStatement.as_view(), name='collections-statement'),
    url(r'^collections_payment/', views.CollectionsPayment.as_view(), name='collections-payment'),
    url(r'^collections_summary/', views.CollectionsSummary.as_view(), name='collections-summary'),
    url(r'^collections_fore_closure/', views.CollectionsForeClosure.as_view(), name='collections-fore-closure'),
    url(r'^collections_sta_card/', views.CollectionsStaCard.as_view(), name='collections-sta-card'),
    url(r'^dedupe_check/', views.DedupeCheck.as_view(), name='dedupe-check'),
    url(r'^dedupe_check_robo/', views.DedupeCheckRobo.as_view(), name='dedupe-check-robo'),
    url(r'^retrieve_references/', views.RetrieveReferences.as_view(), name='retrieve-references'),
    url(r'^fraudCheck/', views.FraudCheckRule.as_view(), name='fraud-check'),
    url(r'^get_active_loans/', views.GetActiveLoanByAppIds.as_view(), name='get-active-loan'),
    url(r'^get_data_custconadd/', views.GetDataCustConAdd.as_view(), name='get-data-custconadd'), #nguyenhuuvinh4
    url(r'^get_blacklisted_accounts/', views.GetBlacklistedAccounts.as_view(), name='get_blacklisted_accounts'),
    url(r'^get_aml_data/', views.getAMLTableData.as_view(), name='get_aml_data'),
    url(r'^get_data_dwh/', views.GetRegisAndActHiFromDWHRequestSerializer.as_view(), name='get_data_dwh'),
]


# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-07-13 05:59
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('fec_dbsync', '0013_auto_20180713_0536'),
    ]

    operations = [
        migrations.AddField(
            model_name='robocollectionfdv',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='robocollectionfdv',
            name='updated_at',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AddField(
            model_name='robocxlview',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='robocxlview',
            name='updated_at',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AddField(
            model_name='roboldaview',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='roboldaview',
            name='updated_at',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AddField(
            model_name='roboltdview',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='roboltdview',
            name='updated_at',
            field=models.DateTimeField(auto_now=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-06-08 08:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('fec_dbsync', '0006_auto_20180607_2053'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='roboidtview',
            name='ACCRUED_DMD_AMT',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='BANK_ID',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='CRYSTALLISED_AMT',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='DEL_FLG',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='DEMAND_IND',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='DMD_DATE',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='DMD_FLOW_ID',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='EI_AMT',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='LATEFEE_STATUS_FLG',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='LCHG_TIME',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='LCHG_USER_ID',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='LDT_CRNCY_CODE',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='LD_FREQ_TYPE',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='RCRE_TIME',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='RCRE_USER_ID',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='SHDL_NUM',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='TOT_REFUND_AMT',
        ),
        migrations.RemoveField(
            model_name='roboidtview',
            name='TS_CNT',
        ),
        migrations.AddField(
            model_name='robochatview',
            name='CHAT_CREATION_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name='robochatview',
            name='CHAT_LAST_CHANGE_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name='robochatview',
            name='GAM_CREATION_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name='robochatview',
            name='GAM_LAST_CHANGE_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name='roboidtview',
            name='GAM_CREATION_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name='roboidtview',
            name='GAM_LAST_CHANGE_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name='roboidtview',
            name='LAST_WAIVER_ADJ_DATE',
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name='roboidtview',
            name='LDT_CREATION_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name='roboidtview',
            name='LDT_LAST_CHANGE_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name='roboidtview',
            name='WAIVED_AMT',
            field=models.BigIntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='roboclelarshview',
            name='CHRG_CMP',
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='roboclelarshview',
            name='INT_CMP',
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='roboclelarshview',
            name='LCHG_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AlterField(
            model_name='roboclelarshview',
            name='PRIN_CMP',
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='roboclelarshview',
            name='RCRE_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AlterField(
            model_name='roboclelarshview',
            name='REP_SHDL_DATE',
            field=models.DateField(null=True),
        ),
        migrations.AlterField(
            model_name='roboclelarshview',
            name='TOTAL_FLOW',
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='roboidtview',
            name='DMD_AMT',
            field=models.BigIntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='roboidtview',
            name='DMD_EFF_DATE',
            field=models.DateField(null=True),
        ),
        migrations.AlterField(
            model_name='roboidtview',
            name='DMD_OVDU_DATE',
            field=models.DateField(null=True),
        ),
        migrations.AlterField(
            model_name='roboidtview',
            name='LAST_ADJ_DATE',
            field=models.DateField(null=True),
        ),
        migrations.AlterField(
            model_name='roboidtview',
            name='TOT_ADJ_AMT',
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='roborepayuplview',
            name='FILE_NAME',
            field=models.TextField(null=True),
        ),
    ]
